/**
 * DevExtreme (exporter/jspdf/export.js)
 * Version: 20.2.1 (build 20263-0330)
 * Build date: Sat Sep 19 2020
 *
 * Copyright (c) 2012 - 2020 Developer Express Inc. ALL RIGHTS RESERVED
 * Read about DevExtreme licensing here: https://js.devexpress.com/Licensing/
 */
"use strict";
exports.Export = void 0;
var _type = require("../../core/utils/type");
var _extend = require("../../core/utils/extend");
var _date = _interopRequireDefault(require("../../localization/date"));

function _interopRequireDefault(obj) {
    return obj && obj.__esModule ? obj : {
        "default": obj
    }
}
var Export = {
    getFullOptions: function(options) {
        var fullOptions = (0, _extend.extend)({}, options);
        if (!((0, _type.isDefined)(fullOptions.jsPDFDocument) && (0, _type.isObject)(fullOptions.jsPDFDocument))) {
            throw Error('The "jsPDFDocument" field must contain a jsPDF instance.')
        }
        if (!((0, _type.isDefined)(fullOptions.jsPDFDocument.autoTable) && (0, _type.isFunction)(fullOptions.jsPDFDocument.autoTable))) {
            throw Error('The "exportDataGrid" method requires a autoTable plugin for jsPDF object.')
        }
        if (!(0, _type.isDefined)(fullOptions.keepColumnWidths)) {
            fullOptions.keepColumnWidths = true
        }
        if (!(0, _type.isDefined)(fullOptions.autoTableOptions)) {
            fullOptions.autoTableOptions = this.getDefaultAutoTableOptions()
        } else {
            if (!(0, _type.isObject)(fullOptions.autoTableOptions)) {
                throw Error('The "autoTableOptions" option must be of object type.')
            }
            fullOptions.autoTableOptions = (0, _extend.extend)(true, {}, this.getDefaultAutoTableOptions(), fullOptions.autoTableOptions)
        }
        return fullOptions
    },
    getDefaultAutoTableOptions: function() {
        return {
            theme: "plain",
            tableLineColor: 149,
            tableLineWidth: .1,
            styles: {
                textColor: 51,
                lineColor: 149,
                lineWidth: 0
            },
            columnStyles: {},
            headStyles: {
                fontStyle: "normal",
                textColor: 149,
                lineWidth: .1
            },
            bodyStyles: {
                lineWidth: .1
            },
            head: [],
            body: []
        }
    },
    "export": function(options) {
        var _this = this;
        var jsPDFDocument = options.jsPDFDocument,
            autoTableOptions = options.autoTableOptions,
            component = options.component,
            keepColumnWidths = options.keepColumnWidths,
            selectedRowsOnly = options.selectedRowsOnly;
        var dataProvider = component.getDataProvider(selectedRowsOnly);
        return new Promise(function(resolve) {
            dataProvider.ready().done(function() {
                var columns = dataProvider.getColumns();
                var styles = dataProvider.getStyles();
                var dataRowsCount = dataProvider.getRowsCount();
                if (keepColumnWidths) {
                    var pdfColumnWidths = _this.getDefaultPdfColumnWidths(autoTableOptions.tableWidth, dataProvider.getColumnsWidths());
                    _this.setColumnWidths(autoTableOptions, pdfColumnWidths)
                }
                for (var rowIndex = 0; rowIndex < dataRowsCount; rowIndex++) {
                    var rowType = null;
                    var row = [];
                    for (var cellIndex = 0; cellIndex < columns.length; cellIndex++) {
                        var cellData = dataProvider.getCellData(rowIndex, cellIndex, true);
                        var cellStyle = styles[dataProvider.getStyleId(rowIndex, cellIndex)];
                        var gridCell = cellData.cellSourceData;
                        var pdfCell = {
                            content: cellData.value,
                            styles: {}
                        };
                        _this.assignCellStyle(pdfCell, gridCell, columns[cellIndex], cellStyle);
                        if (!(0, _type.isDefined)(rowType)) {
                            rowType = gridCell.rowType
                        }
                        row.push(pdfCell)
                    }
                    if ("header" === rowType) {
                        autoTableOptions.head.push(row)
                    }
                    if ("data" === rowType) {
                        autoTableOptions.body.push(row)
                    }
                }
                jsPDFDocument.autoTable(autoTableOptions);
                resolve(jsPDFDocument)
            })
        })
    },
    assignCellStyle: function(pdfCell, gridCell, column, cellStyle) {
        if ("header" === gridCell.rowType) {
            if (column.alignment) {
                pdfCell.styles.halign = column.alignment
            }
        } else {
            if (cellStyle) {
                if (cellStyle.alignment) {
                    pdfCell.styles.halign = cellStyle.alignment
                }
                if (cellStyle.bold) {
                    pdfCell.styles.fontStyle = "bold"
                }
                if (cellStyle.wrapText) {
                    pdfCell.styles.cellWidth = "wrap"
                }
                if ("date" === cellStyle.dataType) {
                    pdfCell.content = _date.default.format(new Date(pdfCell.content), cellStyle.format)
                }
            }
        }
    },
    getDefaultPdfColumnWidths: function(autoTableWidth, columnWidths) {
        if (!(0, _type.isNumeric)(autoTableWidth) || !(0, _type.isDefined)(columnWidths)) {
            return
        }
        var tableWidth = columnWidths.reduce(function(a, b) {
            return a + b
        }, 0);
        return columnWidths.map(function(columnWidth) {
            return autoTableWidth * columnWidth / tableWidth
        })
    },
    setColumnWidths: function(autoTableOptions, pdfColumnWidths) {
        if (!pdfColumnWidths) {
            return
        }
        var columnStyles = autoTableOptions.columnStyles;
        for (var i = 0; i < pdfColumnWidths.length; i++) {
            columnStyles[i] = columnStyles[i] || {};
            columnStyles[i].cellWidth = pdfColumnWidths[i]
        }
    }
};
exports.Export = Export;
